'use strict';
const utils_1 = require("@typescript-eslint/utils");
const typescript_1 = require("typescript");
const rule = utils_1.ESLintUtils.RuleCreator(() => 'https://github.com/amzn/eslint-plugin-no-date-parsing')({
    name: 'no-date-parsing',
    meta: {
        type: 'problem',
        docs: {
            description: 'Disallow `Date` string parsing',
            recommended: 'error'
        },
        messages: {
            mustBeTyped: 'Argument to `Date` must have a TypeScript type.',
            noNewDate: 'Parsing of date strings with `new Date` is strongly discouraged due to browser differences and inconsistencies.',
            noDateParse: 'Parsing of date strings with `Date.parse` is strongly discouraged due to browser differences and inconsistencies.'
        },
        schema: []
    },
    defaultOptions: [],
    create: (context) => {
        const { program, esTreeNodeToTSNodeMap } = utils_1.ESLintUtils.getParserServices(context);
        const checker = program.getTypeChecker();
        function checkConstructor(node) {
            const tsNode = esTreeNodeToTSNodeMap.get(node.arguments[0]);
            const nodeType = checker.getTypeAtLocation(tsNode);
            const constrained = checker.getBaseConstraintOfType(nodeType);
            const type = constrained || nodeType;
            if (type.flags & typescript_1.TypeFlags.Any || type.flags & typescript_1.TypeFlags.Unknown) {
                context.report({
                    node,
                    messageId: 'mustBeTyped'
                });
            }
            else if (!(type.flags & typescript_1.TypeFlags.NumberLike)) {
                context.report({
                    node,
                    messageId: 'noNewDate'
                });
            }
        }
        function checkDateParse(node) {
            context.report({
                node,
                messageId: 'noDateParse'
            });
        }
        return {
            'CallExpression[callee.name="Date"][arguments.length!=0]': checkConstructor,
            'NewExpression[callee.name="Date"][arguments.length!=0]': checkConstructor,
            'CallExpression[callee.object.name="Date"][callee.property.name="parse"]': checkDateParse
        };
    }
});
module.exports = rule;
